package com.qmth.cqb.paper.service.export;

import java.io.File;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;


import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.common.dto.question.enums.QuesStructType;
import cn.com.qmth.examcloud.common.uac.entity.AccessUser;

import com.qmth.cqb.paper.dto.PaperDetailExp;
import com.qmth.cqb.paper.dto.PaperDetailUnitExp;
import com.qmth.cqb.paper.dto.PaperExp;
import com.qmth.cqb.paper.model.ExportStructure;
import com.qmth.cqb.paper.model.ExtractConfig;
import com.qmth.cqb.paper.model.QuestionTypeNum;
import com.qmth.cqb.utils.CommonUtils;
import com.qmth.cqb.utils.enums.ExamFileType;
import com.qmth.cqb.utils.enums.ExportType;
import com.qmth.cqb.utils.word.DocxProcessUtil;

/**
 * 
 * @author  	chenken
 * @date    	2017年7月31日 上午9:21:51
 * @company 	QMTH
 * @description 电子科技大学导出、上传文件service
 */
@Service("dzkdExportPaperService")
public class DzkdExportPaperService extends ExportPaperAbstractService {
	
	public void downloadPaper(String paperId,String zipFileName)throws Exception {
		PaperExp paperExp = initPaperExpService.initPaperExp(paperId);
		setSplitPaperDetailExp(paperExp,paperExp.getPaperDetails());
		if(paperExp!=null){
			String paperfileName = paperExp.getCourseNo()+"_"+ExamFileType.PAPER.getName()+DOCX_SUFFIX;
			File file = new File(TEMP_FILE_EXP+File.separator+zipFileName+File.separator+paperfileName);
			DocxProcessUtil.exportWordNew(paperExp, file,DZKD_TEMPLATE_PAPER);
			DocxProcessUtil.processImage(zipFileName+File.separator+paperfileName,getPkgList(paperId));
		}
	}
	
	public void downloadPaperAnswer(String paperId,String zipFileName) throws Exception {
		PaperExp paperExp = initPaperExpService.initPaperExp(paperId);
		if(paperExp!=null){
			String answerFileName = paperExp.getCourseNo()+"_"+ExamFileType.ANSWER.getName()+DOCX_SUFFIX;
			File file = new File(TEMP_FILE_EXP+File.separator+zipFileName+File.separator+answerFileName);
			DocxProcessUtil.exportWordNew(paperExp,file,DZKD_TEMPLATE_ANSWER);
			DocxProcessUtil.processImage(zipFileName+File.separator+answerFileName,getPkgList(paperId));
		}
	}
	
	/**
	 * 设置大题标题,并将大题分类  主观题    客观题
	 * @param paperDetailExps
	 */
	public void setExpDtoTitle(List<PaperDetailExp> paperDetailExps) {
		int firstBlank = 0;
		int lastBlank = 0;
		for(PaperDetailExp paperDetailExp:paperDetailExps){
			String title = "";
			String titleDetail = null;
			String totalScore = BigDecimal.valueOf(paperDetailExp.getScore()).stripTrailingZeros().toPlainString();
			QuesStructType type = paperDetailExp.getPaperDetailUnits().get(0).getQuestionType();
			String questionScore = initPaperExpService.checkPaperDetailUnitScore(paperDetailExp);
			String scoreString = questionScore==null?"":"每小题"+questionScore+"分，";
			if (type == QuesStructType.SINGLE_ANSWER_QUESTION) {
				paperDetailExp.setName("单项选择题");
				title = "（本大题共" + paperDetailExp.getUnitCount() + "小题，"
						+ scoreString + "共" + totalScore + "分）";
				titleDetail = "在每小题列出的备选项中只有一个符号合题目要求的，请将其选出并将“答题卡”的相应代码涂黑，错涂、多涂或未涂均无分";
			} else if (type == QuesStructType.MULTIPLE_ANSWER_QUESTION) {
				paperDetailExp.setName("多项选择题");
				title = "（本大题共" + paperDetailExp.getUnitCount() + "小题，"
						+ scoreString + "共" + totalScore+ "分）";
				titleDetail = "在每小题列出的备选项中有一个或多个符号合题目要求的，请将符合项选出并将“答题卡”的相应代码涂黑";
			} else if (type == QuesStructType.BOOL_ANSWER_QUESTION) {
				paperDetailExp.setName("判断题");
				title = "（本大题共" + paperDetailExp.getUnitCount() + "小题，"
						+ scoreString + "共" + totalScore+ "分。"
						+ "正确的填涂A、错误填涂B。错涂、多涂或未涂均无分）";
			} else if (type == QuesStructType.FILL_BLANK_QUESTION) {
				paperDetailExp.setName("填空题");
				//计算填空空格数量
				int blanksNumbers = blanksNumbers(paperDetailExp);
				//设置填空题题干中 开始空格  和  结束空格
				lastBlank = firstBlank +  blanksNumbers;
				questionScore = checkPaperDetailUnitblanksScore(paperDetailExp);
				scoreString = questionScore==null?"":"每空"+questionScore+"分，";
				title = "（本大题共" + blanksNumbers + "个空格，将每空答在答题卡 “" + String.valueOf(firstBlank + 1)
						+ "——" + String.valueOf(lastBlank) + "”相应的序号上。"+ scoreString + "共" + totalScore+ "分。)";
				firstBlank = lastBlank;
			} else {
				paperDetailExp.setName(type.getName());
				title = "（本大题共"+paperDetailExp.getUnitCount()+"小题，"+scoreString+"共"+totalScore+"分）";
			}
			paperDetailExp.setTitle(title);
			paperDetailExp.setTitleDetail(titleDetail);
		}
	}
	
	/**
	 * 计算每个大题所有填空题空格数量
	 * @param paperDetailExp
	 * @return
	 */
	private int blanksNumbers(PaperDetailExp paperDetailExp) {
		List<PaperDetailUnitExp> paperDetailUnitExps = paperDetailExp.getPaperDetailUnits();
		int blanksNumbers = 0;
		for(PaperDetailUnitExp paperDetailUnitExp:paperDetailUnitExps){
			String str = paperDetailUnitExp.getQuestion().getQuesAnswer();
			if(str.indexOf("##")>-1){
				//将所有（ 替换为空
				String str1 = str.replaceAll("##", "");
				blanksNumbers += ((str.length() - str1.length())/2+1);
			}else {
				blanksNumbers += 1;
			}
		}
		return blanksNumbers;
	}
	
	/**
	 * 计算每个小题所有填空题空格数量
	 * @param paperDetailExp
	 * @return
	 */
	private int blanksNumbers(PaperDetailUnitExp unitExp) {
		int blanksNumbers = 0;
		String str = unitExp.getQuestion().getQuesAnswer();
		if(str.indexOf("##")>-1){
			//将所有（ 替换为空
			String str1 = str.replaceAll("##", "");
			blanksNumbers += ((str.length() - str1.length())/2+1);
		}else {
			blanksNumbers += 1;
		}
		return blanksNumbers;
	}
	
	/**
	 * 校验大题下的每个填空题空格分数是否一样
	 * @param paperDetailExp
	 * @return  如果分数一样，返回分数
	 * 			如果分数不一样，返回 null
	 */
	protected String checkPaperDetailUnitblanksScore(PaperDetailExp paperDetailExp) {
		List<PaperDetailUnitExp> paperDetailUnitExps = paperDetailExp.getPaperDetailUnits();
		Set<Double> scoreSet = new HashSet<Double>();
		for (PaperDetailUnitExp unitExp : paperDetailUnitExps) {
			int blanksNumbers = blanksNumbers(unitExp);
			double everyBlank = unitExp.getScore() / blanksNumbers;
			scoreSet.add(everyBlank);
		}
		if(scoreSet.size() == 1){
			return scoreSet.iterator().next()+"";
		}
		return null;
	}
	
	 /**
     * 添加主观题与客观题
     * @param paperDetailExps
     */
    public void setSplitPaperDetailExp(PaperExp PaperExp,List<PaperDetailExp> paperDetailExps) {
		// 客观题
		List<PaperDetailExp> objectiveDetails = new ArrayList<PaperDetailExp>();
		// 主观题
		List<PaperDetailExp> subjectiveDetails = new ArrayList<PaperDetailExp>();
		for(PaperDetailExp paperDetailExp:paperDetailExps){
			Long type = paperDetailExp.getSortNumber();
			if(type < 5L){
				objectiveDetails.add(paperDetailExp);
			}else {
				subjectiveDetails.add(paperDetailExp);
			}
		}
		PaperExp.setObjectiveDetails(objectiveDetails);
		PaperExp.setSubjectiveDetails(subjectiveDetails);
		//主观题   客观题的标题显示
		showTitle(PaperExp);
	}
	
	/**
	 * 主观题   客观题的标题显示
	 * @param PaperExp
	 */

	private void showTitle(PaperExp paperExp) {
		String title = "";
		List<PaperDetailExp> objectiveDetails = paperExp.getObjectiveDetails();
		List<PaperDetailExp> subjectiveDetails = paperExp.getSubjectiveDetails();
		//设置客观题的标题
		if(objectiveDetails != null && objectiveDetails.size()>0){
			Double score = 0.0; 
			for(PaperDetailExp objectiveDetail:objectiveDetails){
				score += objectiveDetail.getScore();
			}
			//判断是否有主观题
			if(subjectiveDetails == null || subjectiveDetails.size() < 1){
				title = "客观题（总分" + score + "分）";
				paperExp.setObjectiveTitle(title);
			}else {
				title = "第一部分"+ "    " +"客观题（总分" + score + "分）";
			}
			paperExp.setObjectiveTitle(title);
		}
		if(subjectiveDetails != null && subjectiveDetails.size()>0){
			Double score = 0.0; 
			for(PaperDetailExp objectiveDetail:subjectiveDetails){
				score += objectiveDetail.getScore();
			}
			//判断是否有客观题
			if(objectiveDetails == null || objectiveDetails.size() < 1){
				title = "主观题（总分" + score + "分）";
				paperExp.setSubjectiveTitle(title);
			}else {
				title = "第二部分"+ "    " +"主观题（总分" + score + "分）";
			}
			paperExp.setSubjectiveTitle(title);
		}
	}
	


	
	@Override
    public void uploadFile(ExtractConfig extractConfig,String paperId,ExportStructure exportStructure,AccessUser accessUser) throws Exception{
    	PaperExp paperExp = initPaperExpService.initPaperExp(paperId);
        if (paperExp!=null) {
        	String currNum = CommonUtils.getCurNum();
        	//没有试卷结构导出设置
        	if(exportStructure==null){
        		//上传试卷
        		uploadPaperOrAnswerFile(paperExp,extractConfig,accessUser,currNum,DZKD_TEMPLATE_PAPER,ExamFileType.PAPER);
	        	//上传答案
        		uploadPaperOrAnswerFile(paperExp,extractConfig,accessUser,currNum,DZKD_TEMPLATE_ANSWER,ExamFileType.ANSWER);
        	}else if(exportStructure!=null&&exportStructure.getExportType()==ExportType.NORMAL){
        		List<QuestionTypeNum> questionTypeNums = exportStructure.getQuestionTypeNums();
        		//检查试卷中客观题的数量是否大于试卷导出设置中的设置的数量
        		checkObjectiveDetailsNum(paperExp,questionTypeNums);
	    		//上传试卷
        		uploadPaperOrAnswerFile(paperExp,extractConfig,accessUser,currNum,DZKD_TEMPLATE_PAPER,ExamFileType.PAPER);
	        	//上传答案
        		uploadPaperOrAnswerFile(paperExp,extractConfig,accessUser,currNum,DZKD_TEMPLATE_ANSWER,ExamFileType.ANSWER);
				//上传试卷结构   不能在上传试卷和答案之前
        		uploadPaperStructure(paperExp,extractConfig,accessUser,currNum,questionTypeNums);
        	}else if(exportStructure!=null&&exportStructure.getExportType()==ExportType.ONLINE){
        		//上传机考JSON文件
        		uploadComputerTestFile(extractConfig,accessUser);
        	}
        }
    }

}
